#ifndef WINDOW_H
#define WINDOW_H

#include <ctime>
#include <GL/glew.h>
#include <GLFW/glfw3.h>

#include "tori.h"
#include "skybox.h"
#include "plane.h"
#include "pointlightprogram.h"
#include "texturecameraprogram.h"
#include "texture.h"
#include "cubetexture.h"
#include "matma.h"

class Window{
 public:
    Window(const char*, int, int);
    void Initialize(int major_gl_version, int minor_gl_version);
    void Resize(int new_width, int new_height);
    void KeyEvent(int key, int scancode, int action, int mods);
    void MouseScroll(double xoffset, double yoffset);
    void MouseButton(int button, int action, int mods);
    void MouseMove(double x_pos, double y_pos);
    void Run(void);
    operator GLFWwindow*(){return window_;}
 private:
    int width_;
    int height_;
    const char* title_;
    GLFWwindow* window_;

    Tori tori_;
    Plane plane_;
    SkyBox sky_;

    Texture color_texture_;
    Texture ice_texture_;
    CubeTexture sky_texture_;

    PointLightProgram point_program_;
    TextureCameraProgram sky_program_;

    int x_origin_;
    int y_origin_;

    clock_t last_time_;

    Mat4 view_matrix_;
    Mat4 projection_matrix_;

    void InitTextures();
    void InitModels();
    void InitPrograms();

    void SetViewMatrix();
    void SetProjectionMatrix();

    void InitGlfwOrDie(int major_gl_version, int minor_gl_version);
    void InitGlewOrDie();
   
};


#endif // WINDOW_H

